import Axios from "axios";

import { RequestMapDelete, ResponseMapDelete } from "models/api-delete";
import { RequestMapGet, ResponseMapGet } from "models/api-get";
import { RequestMapPost, ResponseMapPost } from "models/api-post";
import { RequestMapPut, ResponseMapPut } from "models/api-put";
import { BasicType } from "models/basic-type";
import replaceQueryParams from "utils/replace-query-params";

/**
 * Replace the baseUrl with your API url.
 *
 * Since the API functions are written with generics here is an turorial article:
 * https://www.digitalocean.com/community/tutorials/how-to-use-generics-in-typescript
 */

const API = {
  url: "https://jsonplaceholder.typicode.com",
};

const api = Axios.create({
  baseURL: API.url,
});

const getParams = (path: string, queryData?: Record<string, BasicType>) => {
  let apiPath = path;
  if (queryData) {
    const { newPath } = replaceQueryParams(path, queryData);
    apiPath = newPath;
  }
  if (apiPath.indexOf("{") !== -1) {
    throw new Error(`Missing param for "${path}"`);
  }
  return apiPath;
};

/**
 * GET request
 */
const get = async <P extends keyof RequestMapGet, D extends RequestMapGet[P]>(
  path: P,
  data: D
) => {
  const apiPath = getParams(path, data);
  const response = await api.get<ResponseMapGet[P]>(apiPath);
  return response.data;
};
/**
 * POST request
 */
const post = async <
  P extends keyof ResponseMapPost,
  D extends RequestMapPost[P]
>(
  path: P,
  data: D
) => {
  const apiPath = getParams(path, data);
  const response = await api.post<ResponseMapPost[P]>(apiPath, data);
  return response.data;
};

/**
 * PUT request
 */
const put = async <P extends keyof ResponseMapPut, D extends RequestMapPut[P]>(
  path: P,
  data: D
) => {
  const apiPath = getParams(path, data);
  const response = await api.put<ResponseMapPut[P]>(apiPath, data);
  return response.data;
};

/**
 * DELETE request
 */
const remove = async <
  P extends keyof ResponseMapDelete,
  D extends RequestMapDelete[P]
>(
  path: P,
  data: D
) => {
  const apiPath = getParams(path, data);
  const response = await api.delete<ResponseMapDelete[P]>(apiPath);
  return response.data;
};

export default {
  get,
  post,
  put,
  remove,
};
