import React, { ReactElement, useRef, useState } from "react";
import { FlatList, Text, Modal, View, Pressable } from "react-native";
import Chevron from "svg/chevron.svg";

import styles from "./dropdown.style";

type Item = { label: string; value: string };

interface Props {
  label?: string;
  data: Item[];
  initialValue?: Item;
  onSelect: (item: Item) => void;
}

const Dropdown = ({ label, data, initialValue, onSelect }: Props) => {
  const $ = styles();
  const dropDownRef = useRef<View>(null);
  const [visible, setVisible] = useState(false);
  const [selected, setSelected] = useState<Item | undefined>(initialValue);

  const [dropdownWidth, setDropdownWidth] = useState(0);
  const [dropdownTop, setDropdownTop] = useState(0);
  const [dropdownX, setDropdownX] = useState(0);

  const openDropdown = (): void => {
    if (dropDownRef.current) {
      dropDownRef.current.measure((_fx, _fy, w, h, px, py) => {
        setDropdownTop(py + h + 1);
        setDropdownWidth(w);
        setDropdownX(px);
      });
    }
    setVisible(true);
  };

  const toggleDropdown = (): void => {
    if (visible) {
      setVisible(false);
    } else {
      openDropdown();
    }
  };

  const onItemPress = (item: Item): void => {
    setSelected(item);
    onSelect(item);
    setVisible(false);
  };

  const renderItem = ({ item }: any): ReactElement<any, any> => (
    <Pressable style={$.item} onPress={() => onItemPress(item)}>
      <Text style={$.label}>{item.label}</Text>
    </Pressable>
  );

  const renderDropdown = () => {
    return (
      <Modal visible={visible} transparent animationType="none">
        <Pressable style={$.overlay} onPress={() => setVisible(false)}>
          <View
            style={[
              $.dropdown,
              { top: dropdownTop, left: dropdownX, width: dropdownWidth },
            ]}
          >
            <FlatList
              data={data}
              renderItem={renderItem}
              keyExtractor={(_item, index) => index.toString()}
            />
          </View>
        </Pressable>
      </Modal>
    );
  };

  return (
    <Pressable ref={dropDownRef} style={$.selector} onPress={toggleDropdown}>
      {renderDropdown()}
      <Text style={$.label}>{(selected && selected.label) || label}</Text>
      <Chevron
        fill={"gray"}
        style={[
          $.chevron,
          { transform: [{ rotate: visible ? "270deg" : "90deg" }] },
        ]}
      />
    </Pressable>
  );
};

export default Dropdown;
